/*
  GC audio demo by CrowTRobo
  July 12, 2003

  Demonstration of playing audio. 
*/

#include "mygcn.h"
#include "sega.h"
#include "notify.h"

PAD pad;

#define AUDIO_DMA_STARTH    *(volatile u16*)(0xCC005030)
#define AUDIO_DMA_STARTL    *(volatile u16*)(0xCC005032)
#define AUDIO_DMA_LENGTH    *(volatile u16*)(0xCC005036)
#define AUDIO_STREAM_STATUS *(volatile u32*)(0xCC006C00)

/*
  I don't use this register in the demo, but it contains the amount
  of bytes left to transfer/play. So by checking if it's 0, you can
  see if the current sample is done.
*/

#define AUDIO_DMA_LEFT      *(volatile u16*)(0xCC00503a)

/*
  Loads the high and low address registers
  Loads the length, divided by 32, into the length register. If the high
  bit is already set in that register, meaning sound is playing, keep it
  set.
*/

#define LoadSample(addr, len) AUDIO_DMA_STARTH = addr >> 16;       \
                              AUDIO_DMA_STARTL = addr & 0xffff;    \
                              AUDIO_DMA_LENGTH = (AUDIO_DMA_LENGTH & 0x8000) | (len / 32)

/* Set the high bit of the length register */

#define StartSample()  AUDIO_DMA_LENGTH |= 0x8000

/* Clear the high bit of the length register */

#define StopSample()   AUDIO_DMA_LENGTH &= 0x7fff

/* Set the 6th bit of the stream status register */

#define SetFreq32KHz() AUDIO_STREAM_STATUS |= 0x40

/* Clear the 6th bit of the stream status register */

#define SetFreq48KHz() AUDIO_STREAM_STATUS &= 0xffffffbf

int main() {

  u32 *fb = (u32*)0xc0500000;
  u32 playing = 0;

  VIDEO_Init(VIDEO_640X480_NTSC_YUV16);
  VIDEO_SetFrameBuffer(VIDEO_FRAMEBUFFER_BOTH, (u32)fb);
  RENDER_ClearFrameBuffer(fb, COLOR_WHITE);
  DEBUG_InitText(fb, COLOR_BLACK);

  while (1) {
    RENDER_ClearFrameBuffer(fb, COLOR_CREAM);
    DEBUG_Print(30, 60, "Audio demo by CrowTRobo");
    DEBUG_Print(30, 90, "Press A to play 32KHz sample");
    DEBUG_Print(30, 110, "Press B to play 48KHz sample");
    DEBUG_Print(30, 130, "Press X to set freq to 32KHz");
    DEBUG_Print(30, 150, "Press Y to set freq to 48KHz");
    DEBUG_Print(30, 170, "Press Z to stop sample");
    
    if (AUDIO_STREAM_STATUS & 0x40)
      DEBUG_Print(30, 200, "Frequency = 32KHz");
    else
      DEBUG_Print(30, 200, "Frequency = 48KHz");

    PAD_ReadState(&pad, PAD_CHANNEL_0);

    if (pad.Digital.A) {
      LoadSample((u32)sega, 209344);

      if (!playing)
        StartSample();
    }
    else if (pad.Digital.B) {
      LoadSample((u32)notify, 215936);

      if (!playing)
        StartSample();
    }
    else if (pad.Digital.Z) {
      StopSample();
      playing = 0;
    }
    else if (pad.Digital.Y) 
      SetFreq48KHz();
    else if (pad.Digital.X)
      SetFreq32KHz();

    VIDEO_WaitVSync();
  }

  return 0;
}


